@@ -55,7 +55,12 @@ class ChangeHandler(FileSystemEventHandler):
         if event.is_directory:
             tag_name = path.name
             parent_tag_name = path.parent.name
-            print("Removing parent tag association for:", tag_name, "Parent:", parent_tag_name)
+            print(
+                "Removing parent tag association for:",
+                tag_name,
+                "Parent:",
+                parent_tag_name,
+            )
             with Persistor() as db:
                 db.remove_parent_tag_from_tag(parent_tag_name, tag_name)
         else:  # Symlink
