@@ -1,7 +1,9 @@
 import os
 import sqlite3
+from typing import List
 from pathlib import Path
 import filetype  # type: ignore
+from fuzzywuzzy import process  # type: ignore
 
 
 class Persistor:
@@ -15,7 +17,7 @@ class Persistor:
         self.hypertagfs_name = "HyperTagFS"
         self.ignore_list_name = "ignore_list"
         self.ignore_list = ["node_modules", "__pycache__"]
-        self.conn = sqlite3.connect(path)
+        self.conn = sqlite3.connect(str(path))
         self.c = self.conn.cursor()
         self.file_groups_types = {
             "Images": ["jpg", "png", "svg", "tif", "ico", "icns"],
@@ -89,18 +91,30 @@ class Persistor:
             CREATE TABLE IF NOT EXISTS
             files(
                 file_id INTEGER PRIMARY KEY,
-                name TEXT,
-                path TEXT UNIQUE
+                name TEXT NOT NULL,
+                path TEXT UNIQUE NOT NULL,
+                embedding_vector TEXT,
+                clean_text TEXT
             )
             """
         )
 
+        # Migrate old files table
+        try:
+            self.c.execute("ALTER TABLE files ADD COLUMN embedding_vector TEXT")
+        except sqlite3.OperationalError:
+            pass
+        try:
+            self.c.execute("ALTER TABLE files ADD COLUMN clean_text TEXT")
+        except sqlite3.OperationalError:
+            pass
+
         self.c.execute(
             """
             CREATE TABLE IF NOT EXISTS
             tags(
                 tag_id INTEGER PRIMARY KEY,
-                name TEXT UNIQUE
+                name TEXT UNIQUE NOT NULL
             )
             """
         )
@@ -231,6 +245,52 @@ class Persistor:
         tag_id = self.get_tag_id_by_name(name)
         return tag_id
 
+    def add_file_embedding_vector(self, file_path: str, embedding_vector: str):
+        self.c.execute(
+            """
+            UPDATE OR IGNORE files
+            SET
+                embedding_vector = ?
+            WHERE
+                path = ?
+            """,
+            [str(embedding_vector), file_path],
+        )
+
+    def get_file_embedding_vectors(self, file_paths):
+        # Returns list of (file_id, embedding_vector)
+        file_embedding_vectors = []
+        for file_path in file_paths:
+            self.c.execute(
+                """
+                SELECT path, embedding_vector
+                FROM files
+                WHERE embedding_vector IS NOT NULL AND
+                    path = ?
+                """,
+                (str(file_path),),
+            )
+            result = self.c.fetchone()
+            if result:
+                path, embedding_vector = result
+                if embedding_vector != "nan":
+                    file_embedding_vectors.append((path, embedding_vector))
+        return file_embedding_vectors
+
+    def get_indexed_file_paths(self, show_path=True):
+        if show_path:
+            head = "SELECT path"
+        else:
+            head = "SELECT name"
+        self.c.execute(head + " FROM files WHERE embedding_vector is NOT NULL")
+        data = [e[0] for e in self.c.fetchall()]
+        return data
+
+    def get_unindexed_file_paths(self) -> List[str]:
+        self.c.execute("SELECT path FROM files WHERE embedding_vector is NULL")
+        data = [str(e[0]) for e in self.c.fetchall()]
+        return data
+
     def get_files(self, show_path):
         if show_path:
             self.c.execute("SELECT path FROM files")
@@ -272,6 +332,10 @@ class Persistor:
         self.c.execute("SELECT file_id FROM files WHERE name LIKE ?", [name])
         return self.c.fetchone()[0]
 
+    def get_file_path_by_id(self, file_id: int):
+        self.c.execute("SELECT file_path FROM files WHERE file_id = ?", [file_id])
+        return self.c.fetchone()[0]
+
     def remove_parent_tag_from_tag(self, parent_tag_name: str, tag_name: str):
         try:
             parent_tag_id = self.get_tag_id_by_name(parent_tag_name)
@@ -286,6 +350,29 @@ class Persistor:
         )
         self.conn.commit()
 
+    def get_add_clean_text_of_file(self, file_path: str):
+        self.c.execute(
+            """
+            SELECT clean_text FROM files WHERE path LIKE ?
+            """,
+            (file_path,),
+        )
+        data = self.c.fetchone()[0]
+        return data
+
+    def add_clean_text_to_file(self, file_path: str, clean_text: str):
+        self.c.execute(
+            """
+            UPDATE files
+            SET
+                clean_text = ?
+            WHERE
+                path = ?
+            """,
+            [clean_text, file_path],
+        )
+        self.conn.commit()
+
     def add_parent_tag_to_tag(self, parent_tag_name: str, tag_name: str):
         self.c.execute(
             """
@@ -404,7 +491,7 @@ class Persistor:
             """
             SELECT tt.parent_tag_id
             FROM tags as t, tags_tags as tt
-            WHERE 
+            WHERE
                 tt.children_tag_id = t.tag_id AND
                 t.name LIKE ?
             """,
@@ -418,7 +505,7 @@ class Persistor:
             """
             SELECT t.name
             FROM tags as t, tags_files as tf, files as f
-            WHERE 
+            WHERE
                 t.tag_id = tf.tag_id AND
                 tf.file_id = f.file_id AND
                 f.name LIKE ?
@@ -442,7 +529,22 @@ class Persistor:
         data = self.c.fetchall()
         return data
 
-    def get_files_by_tag(self, tag_name, show_path):
+    def get_files_by_tag(self, tag_name, show_path, fuzzy, verbose=False):
+        if fuzzy:
+            matches = process.extract(tag_name, self.get_tags(), limit=5)
+            best_match = None, None
+            best_dist = float("inf")
+            for match, ratio in matches:
+                length_dist = abs(len(match) - len(tag_name)) + 1
+                length_overlap = len(set(tag_name).intersection(match))
+                dist = ((length_dist ** 0.5) / ((length_overlap + 1) ** 3)) / ratio
+                if dist < best_dist:
+                    best_dist = dist
+                    best_match = match, ratio
+
+            tag_name, ratio = best_match
+            if verbose:
+                print(f"Fuzzy matched: {tag_name} ({best_dist*100:.5f})")
         if show_path:
             select = "SELECT f.path"
         else:
